@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011 Red Hat, Inc.
+ * Copyright (C) 2011, 2012 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -106,19 +106,9 @@ build_authorization_uri (GoaOAuth2Provider  *provider,
 static const gchar *
 get_authorization_uri (GoaOAuth2Provider *provider)
 {
-  return "https://m.facebook.com/dialog/oauth";
+  return "https://www.facebook.com/dialog/oauth";
 }
 
-
-static const gchar *
-get_token_uri (GoaOAuth2Provider *provider)
-{
-  /* Not used in client-side auth flow, since the access token is obtained
-   * directly from the authorization phase (get_authorization_uri()) */
-  return NULL;
-}
-
-
 static const gchar *
 get_redirect_uri (GoaOAuth2Provider *provider)
 {
@@ -152,6 +142,12 @@ get_client_secret (GoaOAuth2Provider *provider)
   return NULL;
 }
 
+static const gchar *
+get_authentication_cookie (GoaOAuth2Provider *provider)
+{
+  return "c_user";
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gchar *
@@ -218,7 +214,7 @@ get_identity_sync (GoaOAuth2Provider  *provider,
       g_set_error (error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED,
-                   _("Didn't find username member in JSON data"));
+                   _("Didn't find id member in JSON data"));
       goto out;
     }
   presentation_identity = g_strdup (json_object_get_string_member (json_object, "email"));
@@ -227,7 +223,7 @@ get_identity_sync (GoaOAuth2Provider  *provider,
       g_set_error (error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED,
-                   _("Didn't find name member in JSON data"));
+                   _("Didn't find email member in JSON data"));
       goto out;
     }
 
@@ -259,20 +255,27 @@ build_object (GoaProvider         *provider,
               GoaObjectSkeleton   *object,
               GKeyFile            *key_file,
               const gchar         *group,
+              gboolean             just_added,
               GError             **error)
 {
+  GoaAccount *account;
   GoaChat *chat = NULL;
   gboolean chat_enabled;
   gboolean ret = FALSE;
 
+  account = NULL;
+
   /* Chain up */
   if (!GOA_PROVIDER_CLASS (goa_facebook_provider_parent_class)->build_object (provider,
                                                                               object,
                                                                               key_file,
                                                                               group,
+                                                                              just_added,
                                                                               error))
     goto out;
 
+  account = goa_object_get_account (GOA_OBJECT (object));
+
   /* Chat */
   chat = goa_object_get_chat (GOA_OBJECT (object));
   chat_enabled = g_key_file_get_boolean (key_file, group, "ChatEnabled", NULL);
@@ -290,12 +293,22 @@ build_object (GoaProvider         *provider,
         goa_object_skeleton_set_chat (object, NULL);
     }
 
+  if (just_added)
+    {
+      goa_account_set_chat_disabled (account, !chat_enabled);
+      g_signal_connect (account,
+                        "notify::chat-disabled",
+                        G_CALLBACK (goa_util_account_notify_property_cb),
+                        "ChatEnabled");
+    }
+
   ret = TRUE;
 
  out:
   if (chat != NULL)
     g_object_unref (chat);
-
+  if (account != NULL)
+    g_object_unref (account);
   return ret;
 }
 
@@ -312,14 +325,21 @@ show_account (GoaProvider         *provider,
               GoaClient           *client,
               GoaObject           *object,
               GtkBox              *vbox,
-              GtkTable            *table)
+              GtkGrid             *left,
+              GtkGrid             *right)
 {
   /* Chain up */
-  GOA_PROVIDER_CLASS (goa_facebook_provider_parent_class)->show_account (provider, client, object, vbox, table);
+  GOA_PROVIDER_CLASS (goa_facebook_provider_parent_class)->show_account (provider,
+                                                                         client,
+                                                                         object,
+                                                                         vbox,
+                                                                         left,
+                                                                         right);
 
-  goa_util_add_row_editable_label_from_keyfile (table, object, _("Email Address"), "PresentationIdentity", FALSE);
-  goa_util_add_heading (table, _("Use this account for"));
-  goa_util_add_row_switch_from_keyfile (table, object, _("Chat"), "ChatEnabled");
+  goa_util_add_row_switch_from_keyfile_with_blurb (left, right, object,
+                                                   _("Use for"),
+                                                   "chat-disabled",
+                                                   _("Chat"));
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -352,12 +372,12 @@ goa_facebook_provider_class_init (GoaFacebookProviderClass *klass)
 
   oauth2_class = GOA_OAUTH2_PROVIDER_CLASS (klass);
   oauth2_class->get_authorization_uri    = get_authorization_uri;
-  oauth2_class->get_token_uri            = get_token_uri;
   oauth2_class->build_authorization_uri  = build_authorization_uri;
   oauth2_class->get_redirect_uri         = get_redirect_uri;
   oauth2_class->get_scope                = get_scope;
   oauth2_class->get_client_id            = get_client_id;
   oauth2_class->get_client_secret        = get_client_secret;
+  oauth2_class->get_authentication_cookie = get_authentication_cookie;
   oauth2_class->get_identity_sync        = get_identity_sync;
   oauth2_class->get_use_external_browser = get_use_external_browser;
   oauth2_class->add_account_key_values   = add_account_key_values;
