@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011 Red Hat, Inc.
+ * Copyright (C) 2011, 2012 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,12 +26,14 @@
 #include <stdlib.h>
 
 #include <rest/oauth2-proxy.h>
-#include <libsoup/soup-gnome.h>
+#include <libsoup/soup.h>
 #include <webkit/webkit.h>
 #include <json-glib/json-glib.h>
 
 #include "goalogging.h"
 #include "goaprovider.h"
+#include "goautils.h"
+#include "goawebview.h"
 #include "goaoauth2provider.h"
 
 /**
@@ -235,6 +237,14 @@ goa_oauth2_provider_get_authorization_uri (GoaOAuth2Provider *provider)
   return GOA_OAUTH2_PROVIDER_GET_CLASS (provider)->get_authorization_uri (provider);
 }
 
+/* ---------------------------------------------------------------------------------------------------- */
+
+static const gchar *
+goa_oauth2_provider_get_token_uri_default (GoaOAuth2Provider  *provider)
+{
+  return NULL;
+}
+
 /**
  * goa_oauth2_provider_get_token_uri:
  * @provider: A #GoaOAuth2Provider.
@@ -243,10 +253,13 @@ goa_oauth2_provider_get_authorization_uri (GoaOAuth2Provider *provider)
  * url="http://tools.ietf.org/html/draft-ietf-oauth-v2-15#section-2.2">token
  * endpoint</ulink> used for obtaining an access token.
  *
- * You should not include any parameters in the returned URI.
+ * A token URI is only needed when the OAuth2 provider does not support
+ * a separate client-side flow. In such cases, override
+ * #GoaOAuth2ProviderClass.get_token_uri. You should not include any
+ * parameters in the returned URI.
  *
- * This is a pure virtual method - a subclass must provide an
- * implementation.
+ * This is a virtual method where the default implementation returns
+ * %NULL.
  *
  * Returns: (transfer none): A string owned by @provider - do not free.
  */
@@ -257,6 +270,8 @@ goa_oauth2_provider_get_token_uri (GoaOAuth2Provider *provider)
   return GOA_OAUTH2_PROVIDER_GET_CLASS (provider)->get_token_uri (provider);
 }
 
+/* ---------------------------------------------------------------------------------------------------- */
+
 /**
  * goa_oauth2_provider_get_redirect_uri:
  * @provider: A #GoaOAuth2Provider.
@@ -337,6 +352,26 @@ goa_oauth2_provider_get_client_secret (GoaOAuth2Provider *provider)
   return GOA_OAUTH2_PROVIDER_GET_CLASS (provider)->get_client_secret (provider);
 }
 
+/**
+ * goa_oauth2_provider_get_authentication_cookie:
+ * @provider: A #GoaOAuth2Provider.
+ *
+ * Gets the name of a cookie whose presence indicates that the user has been able to
+ * log in during the authorization step. This is used to modify the embedded web
+ * browser UI that is presented to the user.
+ *
+ * This is a pure virtual method - a subclass must provide an
+ * implementation.
+ *
+ * Returns: (transfer none): A string owned by @provider - do not free.
+ */
+const gchar *
+goa_oauth2_provider_get_authentication_cookie (GoaOAuth2Provider *provider)
+{
+  g_return_val_if_fail (GOA_IS_OAUTH2_PROVIDER (provider), NULL);
+  return GOA_OAUTH2_PROVIDER_GET_CLASS (provider)->get_authentication_cookie (provider);
+}
+
 /**
  * goa_oauth2_provider_get_identity_sync:
  * @provider: A #GoaOAuth2Provider.
@@ -535,6 +570,48 @@ typedef struct
   gchar *presentation_identity;
 } IdentifyData;
 
+static void
+check_cookie (SoupCookie *cookie, gpointer user_data)
+{
+  IdentifyData *data = user_data;
+  GList *children;
+  GList *l;
+  GoaOAuth2Provider *provider = data->provider;
+  GtkDialog *dialog = data->dialog;
+  GtkWidget *action_area;
+  const gchar *auth_cookie;
+  const gchar *name;
+
+  auth_cookie = goa_oauth2_provider_get_authentication_cookie (provider);
+  name = soup_cookie_get_name (cookie);
+  if (g_strcmp0 (auth_cookie, name) != 0)
+    return;
+
+  action_area = gtk_dialog_get_action_area (dialog);
+  children = gtk_container_get_children (GTK_CONTAINER (action_area));
+  for (l = children; l != NULL; l = l->next)
+    {
+      GtkWidget *child = l->data;
+      gtk_container_remove (GTK_CONTAINER (action_area), child);
+    }
+  g_list_free (children);
+}
+
+static void
+on_web_view_document_load_finished (WebKitWebView *web_view, WebKitWebFrame *frame, gpointer user_data)
+{
+  IdentifyData *data = user_data;
+  GSList *slist;
+  SoupCookieJar *cookie_jar;
+  SoupSession *session;
+
+  session = webkit_get_default_session ();
+  cookie_jar = SOUP_COOKIE_JAR (soup_session_get_feature (session, SOUP_TYPE_COOKIE_JAR));
+  slist = soup_cookie_jar_all_cookies (cookie_jar);
+  g_slist_foreach (slist, (GFunc) check_cookie, data);
+  g_slist_free_full (slist, (GDestroyNotify) soup_cookie_free);
+}
+
 static gboolean
 on_web_view_navigation_policy_decision_requested (WebKitWebView             *webView,
                                                   WebKitWebFrame            *frame,
@@ -691,6 +768,7 @@ get_tokens_and_identity (GoaOAuth2Provider  *provider,
 
   memset (&data, '\0', sizeof (IdentifyData));
   data.provider = provider;
+  data.dialog = dialog;
   data.loop = g_main_loop_new (NULL, FALSE);
 
   /* TODO: use oauth2_proxy_build_login_url_full() */
@@ -741,47 +819,25 @@ get_tokens_and_identity (GoaOAuth2Provider  *provider,
     }
   else
     {
-      GtkWidget *scrolled_window;
       GtkWidget *web_view;
-      SoupSession *webkit_soup_session;
-      SoupCookieJar *cookie_jar;
+      GtkWidget *embed;
 
-      webkit_soup_session = webkit_get_default_session ();
-      /* Get the proxy configuration from the GNOME settings */
-      soup_session_add_feature_by_type (webkit_soup_session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
-      /* Set the Accept-Language header automatically */
-      g_object_set (webkit_soup_session, "accept-language-auto", TRUE, NULL);
+      web_view = goa_web_view_new ();
+      embed = goa_web_view_get_view (GOA_WEB_VIEW (web_view));
 
-      /* Ensure we use an empty non-persistent cookie to avoid login
-       * credentials being reused...
-       */
-      soup_session_remove_feature_by_type (webkit_soup_session, SOUP_TYPE_COOKIE_JAR);
-      cookie_jar = soup_cookie_jar_new ();
-      soup_session_add_feature (webkit_soup_session, SOUP_SESSION_FEATURE (cookie_jar));
-      g_object_unref (cookie_jar);
-
-      /* TODO: we might need to add some more web browser UI to make this
-       * work...
-       */
-      web_view = webkit_web_view_new ();
-      webkit_web_view_load_uri (WEBKIT_WEB_VIEW (web_view), url);
-      g_signal_connect (web_view,
+      webkit_web_view_load_uri (WEBKIT_WEB_VIEW (embed), url);
+      g_signal_connect (embed,
+                        "document-load-finished",
+                        G_CALLBACK (on_web_view_document_load_finished),
+                        &data);
+      g_signal_connect (embed,
                         "navigation-policy-decision-requested",
                         G_CALLBACK (on_web_view_navigation_policy_decision_requested),
                         &data);
 
-      scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-      gtk_widget_set_size_request (scrolled_window, 500, 400);
-      gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
-                                           GTK_SHADOW_IN);
-      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-                                      GTK_POLICY_AUTOMATIC,
-                                      GTK_POLICY_AUTOMATIC);
-      gtk_container_add (GTK_CONTAINER (scrolled_window), web_view);
-      gtk_container_add (GTK_CONTAINER (vbox), scrolled_window);
-      gtk_widget_show_all (scrolled_window);
+      gtk_container_add (GTK_CONTAINER (vbox), web_view);
+      gtk_widget_show_all (web_view);
     }
-  data.dialog = dialog;
   gtk_dialog_run (GTK_DIALOG (dialog));
 
   /* we can have either the auth code, with which we'll obtain the token, or
@@ -940,8 +996,6 @@ goa_oauth2_provider_add_account (GoaProvider *_provider,
   gchar *refresh_token;
   gchar *identity;
   gchar *presentation_identity;
-  GList *accounts;
-  GList *l;
   AddData data;
   GVariantBuilder builder;
 
@@ -957,7 +1011,6 @@ goa_oauth2_provider_add_account (GoaProvider *_provider,
   refresh_token = NULL;
   identity = NULL;
   presentation_identity = NULL;
-  accounts = NULL;
 
   memset (&data, '\0', sizeof (AddData));
   data.loop = g_main_loop_new (NULL, FALSE);
@@ -977,34 +1030,12 @@ goa_oauth2_provider_add_account (GoaProvider *_provider,
   /* OK, got the identity... see if there's already an account
    * of this type with the given identity
    */
-  accounts = goa_client_get_accounts (client);
-  for (l = accounts; l != NULL; l = l->next)
-    {
-      GoaObject *object = GOA_OBJECT (l->data);
-      GoaAccount *account;
-      GoaOAuth2Based *oauth2_based;
-      const gchar *identity_from_object;
-
-      account = goa_object_peek_account (object);
-      oauth2_based = goa_object_peek_oauth2_based (object);
-      if (oauth2_based == NULL)
-        continue;
-
-      if (g_strcmp0 (goa_account_get_provider_type (account),
-                     goa_provider_get_provider_type (GOA_PROVIDER (provider))) != 0)
-        continue;
-
-      identity_from_object = goa_account_get_identity (account);
-      if (g_strcmp0 (identity_from_object, identity) == 0)
-        {
-          g_set_error (&data.error,
-                       GOA_ERROR,
-                       GOA_ERROR_ACCOUNT_EXISTS,
-                       _("There is already an account for the identity %s"),
-                       identity);
-          goto out;
-        }
-    }
+  if (!goa_utils_check_duplicate (client,
+                                  identity,
+                                  goa_provider_get_provider_type (GOA_PROVIDER (provider)),
+                                  (GoaPeekInterfaceFunc) goa_object_peek_oauth2_based,
+                                  &data.error))
+    goto out;
 
   /* we want the GoaClient to update before this method returns (so it
    * can create a proxy for the new object) so run the mainloop while
@@ -1045,18 +1076,14 @@ goa_oauth2_provider_add_account (GoaProvider *_provider,
     goto out;
 
  out:
+  /* We might have an object even when data.error is set.
+   * eg., if we failed to store the credentials in the keyring.
+   */
   if (data.error != NULL)
-    {
-      g_propagate_error (error, data.error);
-      g_assert (ret == NULL);
-    }
+    g_propagate_error (error, data.error);
   else
-    {
-      g_assert (ret != NULL);
-    }
+    g_assert (ret != NULL);
 
-  g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
-  g_list_free (accounts);
   g_free (identity);
   g_free (presentation_identity);
   g_free (refresh_token);
@@ -1169,7 +1196,8 @@ goa_oauth2_provider_refresh_account (GoaProvider  *_provider,
 static void
 free_mutex (GMutex *mutex)
 {
-  g_mutex_free (mutex);
+  g_mutex_clear (mutex);
+  g_slice_free (GMutex, mutex);
 }
 
 /**
@@ -1244,7 +1272,8 @@ goa_oauth2_provider_get_access_token_sync (GoaOAuth2Provider  *provider,
   lock = g_object_get_data (G_OBJECT (object), "-goa-oauth2-provider-get-access-token-lock");
   if (lock == NULL)
     {
-      lock = g_mutex_new ();
+      lock = g_slice_new0 (GMutex);
+      g_mutex_init (lock);
       g_object_set_data_full (G_OBJECT (object),
                               "-goa-oauth2-provider-get-access-token-lock",
                               lock,
@@ -1405,6 +1434,7 @@ goa_oauth2_provider_build_object (GoaProvider         *provider,
                                   GoaObjectSkeleton   *object,
                                   GKeyFile            *key_file,
                                   const gchar         *group,
+                                  gboolean             just_added,
                                   GError             **error)
 {
   GoaOAuth2Based *oauth2_based;
@@ -1515,6 +1545,7 @@ goa_oauth2_provider_class_init (GoaOAuth2ProviderClass *klass)
   provider_class->ensure_credentials_sync    = goa_oauth2_provider_ensure_credentials_sync;
 
   klass->build_authorization_uri  = goa_oauth2_provider_build_authorization_uri_default;
+  klass->get_token_uri            = goa_oauth2_provider_get_token_uri_default;
   klass->get_use_external_browser = goa_oauth2_provider_get_use_external_browser_default;
   klass->add_account_key_values   = goa_oauth2_provider_add_account_key_values_default;
 }
