@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011 Red Hat, Inc.
+ * Copyright (C) 2011, 2012 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -64,6 +64,7 @@ struct _GoaOAuth2Provider
  * @get_scope: Virtual function for goa_oauth2_provider_get_scope().
  * @get_client_id: Virtual function for goa_oauth2_provider_get_client_id().
  * @get_client_secret: Virtual function for goa_oauth2_provider_get_client_secret().
+ * @get_authentication_cookie: Virtual function for goa_oauth2_provider_get_authentication_cookie().
  * @get_identity_sync: Virtual function for goa_oauth2_provider_get_identity_sync().
  * @build_authorization_uri: Virtual function for goa_oauth2_provider_build_authorization_uri().
  * @get_use_external_browser: Virtual function for goa_oauth2_provider_get_use_external_browser().
@@ -77,11 +78,11 @@ struct _GoaOAuth2ProviderClass
 
   /* pure virtual */
   const gchar *(*get_authorization_uri)    (GoaOAuth2Provider  *provider);
-  const gchar *(*get_token_uri)            (GoaOAuth2Provider  *provider);
   const gchar *(*get_redirect_uri)         (GoaOAuth2Provider  *provider);
   const gchar *(*get_scope)                (GoaOAuth2Provider  *provider);
   const gchar *(*get_client_id)            (GoaOAuth2Provider  *provider);
   const gchar *(*get_client_secret)        (GoaOAuth2Provider  *provider);
+  const gchar *(*get_authentication_cookie) (GoaOAuth2Provider  *provider);
   gchar       *(*get_identity_sync)        (GoaOAuth2Provider  *provider,
                                             const gchar        *access_token,
                                             gchar             **out_presentation_identity,
@@ -94,6 +95,7 @@ struct _GoaOAuth2ProviderClass
                                             const gchar        *escaped_redirect_uri,
                                             const gchar        *escaped_client_id,
                                             const gchar        *escaped_scope);
+  const gchar *(*get_token_uri)            (GoaOAuth2Provider  *provider);
   gboolean     (*get_use_external_browser) (GoaOAuth2Provider  *provider);
   void         (*add_account_key_values)   (GoaOAuth2Provider  *provider,
                                             GVariantBuilder    *builder);
@@ -110,6 +112,7 @@ const gchar *goa_oauth2_provider_get_redirect_uri         (GoaOAuth2Provider  *p
 const gchar *goa_oauth2_provider_get_scope                (GoaOAuth2Provider  *provider);
 const gchar *goa_oauth2_provider_get_client_id            (GoaOAuth2Provider  *provider);
 const gchar *goa_oauth2_provider_get_client_secret        (GoaOAuth2Provider  *provider);
+const gchar *goa_oauth2_provider_get_authentication_cookie (GoaOAuth2Provider  *provider);
 gchar       *goa_oauth2_provider_get_identity_sync        (GoaOAuth2Provider  *provider,
                                                            const gchar        *access_token,
                                                            gchar             **out_presentation_identity,
