@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011 Red Hat, Inc.
+ * Copyright (C) 2011, 2012 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,12 +25,14 @@
 #include <stdlib.h>
 
 #include <rest/oauth-proxy.h>
-#include <libsoup/soup-gnome.h>
+#include <libsoup/soup.h>
 #include <webkit/webkit.h>
 #include <json-glib/json-glib.h>
 
 #include "goalogging.h"
 #include "goaprovider.h"
+#include "goautils.h"
+#include "goawebview.h"
 #include "goaoauthprovider.h"
 
 /**
@@ -340,6 +342,26 @@ goa_oauth_provider_get_callback_uri (GoaOAuthProvider *provider)
   return GOA_OAUTH_PROVIDER_GET_CLASS (provider)->get_callback_uri (provider);
 }
 
+/**
+ * goa_oauth_provider_get_authentication_cookie:
+ * @provider: A #GoaOAuthProvider.
+ *
+ * Gets the name of a cookie whose presence indicates that the user has been able to
+ * log in during the authorization step. This is used to modify the embedded web
+ * browser UI that is presented to the user.
+ *
+ * This is a pure virtual method - a subclass must provide an
+ * implementation.
+ *
+ * Returns: (transfer none): A string owned by @provider - do not free.
+ */
+const gchar *
+goa_oauth_provider_get_authentication_cookie (GoaOAuthProvider *provider)
+{
+  g_return_val_if_fail (GOA_IS_OAUTH_PROVIDER (provider), NULL);
+  return GOA_OAUTH_PROVIDER_GET_CLASS (provider)->get_authentication_cookie (provider);
+}
+
 /**
  * goa_oauth_provider_get_identity_sync:
  * @provider: A #GoaOAuthProvider.
@@ -380,6 +402,29 @@ goa_oauth_provider_get_identity_sync (GoaOAuthProvider *provider,
   return GOA_OAUTH_PROVIDER_GET_CLASS (provider)->get_identity_sync (provider, access_token, access_token_secret, out_presentation_identity, cancellable, error);
 }
 
+/**
+ * goa_oauth_provider_parse_request_token_error:
+ * @provider: A #GoaOAuthProvider.
+ * @call: The #RestProxyCall that was used to fetch the request token.
+ *
+ * Tries to parse the headers and payload within @call to provide a
+ * human readable error message in case the request token could not
+ * be fetched.
+ *
+ * This is a pure virtual method - a subclass must provide an
+ * implementation.
+ *
+ * Returns: A human readable error message or %NULL if the cause of the
+ * error could not be determined. The returned string must be freed with
+ * g_free().
+ */
+gchar *
+goa_oauth_provider_parse_request_token_error (GoaOAuthProvider *provider, RestProxyCall *call)
+{
+  g_return_val_if_fail (GOA_IS_OAUTH_PROVIDER (provider), NULL);
+  return GOA_OAUTH_PROVIDER_GET_CLASS (provider)->parse_request_token_error (provider, call);
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gchar *
@@ -514,6 +559,48 @@ typedef struct
   gint session_handle_expires_in;
 } IdentifyData;
 
+static void
+check_cookie (SoupCookie *cookie, gpointer user_data)
+{
+  IdentifyData *data = user_data;
+  GList *children;
+  GList *l;
+  GoaOAuthProvider *provider = data->provider;
+  GtkDialog *dialog = data->dialog;
+  GtkWidget *action_area;
+  const gchar *auth_cookie;
+  const gchar *name;
+
+  auth_cookie = goa_oauth_provider_get_authentication_cookie (provider);
+  name = soup_cookie_get_name (cookie);
+  if (g_strcmp0 (auth_cookie, name) != 0)
+    return;
+
+  action_area = gtk_dialog_get_action_area (dialog);
+  children = gtk_container_get_children (GTK_CONTAINER (action_area));
+  for (l = children; l != NULL; l = l->next)
+    {
+      GtkWidget *child = l->data;
+      gtk_container_remove (GTK_CONTAINER (action_area), child);
+    }
+  g_list_free (children);
+}
+
+static void
+on_web_view_document_load_finished (WebKitWebView *web_view, WebKitWebFrame *frame, gpointer user_data)
+{
+  IdentifyData *data = user_data;
+  GSList *slist;
+  SoupCookieJar *cookie_jar;
+  SoupSession *session;
+
+  session = webkit_get_default_session ();
+  cookie_jar = SOUP_COOKIE_JAR (soup_session_get_feature (session, SOUP_TYPE_COOKIE_JAR));
+  slist = soup_cookie_jar_all_cookies (cookie_jar);
+  g_slist_foreach (slist, (GFunc) check_cookie, data);
+  g_slist_free_full (slist, (GDestroyNotify) soup_cookie_free);
+}
+
 static gboolean
 on_web_view_navigation_policy_decision_requested (WebKitWebView             *webView,
                                                   WebKitWebFrame            *frame,
@@ -569,6 +656,13 @@ on_entry_changed (GtkEditable *editable,
   gtk_dialog_set_response_sensitive (data->dialog, GTK_RESPONSE_OK, sensitive);
 }
 
+static void
+rest_proxy_call_cb (RestProxyCall *call, const GError *error, GObject *weak_object, gpointer user_data)
+{
+  IdentifyData *data = user_data;
+  g_main_loop_quit (data->loop);
+}
+
 static gboolean
 get_tokens_and_identity (GoaOAuthProvider *provider,
                          GtkDialog        *dialog,
@@ -589,6 +683,7 @@ get_tokens_and_identity (GoaOAuthProvider *provider,
   RestProxy *proxy;
   RestProxyCall *call;
   GHashTable *f;
+  GtkWidget *spinner;
   gboolean use_external_browser;
   gchar **request_params;
   guint n;
@@ -611,9 +706,9 @@ get_tokens_and_identity (GoaOAuthProvider *provider,
 
   memset (&data, '\0', sizeof (IdentifyData));
   data.provider = provider;
+  data.dialog = dialog;
   data.loop = g_main_loop_new (NULL, FALSE);
 
-  /* TODO: run in worker thread */
   proxy = oauth_proxy_new (goa_oauth_provider_get_consumer_key (provider),
                            goa_oauth_provider_get_consumer_secret (provider),
                            goa_oauth_provider_get_request_uri (provider), FALSE);
@@ -631,20 +726,33 @@ get_tokens_and_identity (GoaOAuthProvider *provider,
       for (n = 0; request_params[n] != NULL; n += 2)
         rest_proxy_call_add_param (call, request_params[n], request_params[n+1]);
     }
-  if (!rest_proxy_call_sync (call, &data.error))
+  if (!rest_proxy_call_async (call, rest_proxy_call_cb, NULL, &data, &data.error))
     {
       g_prefix_error (&data.error, _("Error getting a Request Token: "));
       goto out;
     }
+
+  spinner = gtk_spinner_new ();
+  gtk_spinner_start (GTK_SPINNER (spinner));
+  gtk_box_pack_start (vbox, spinner, TRUE, TRUE, 0);
+  gtk_widget_show (spinner);
+
+  g_main_loop_run (data.loop);
+  gtk_container_remove (GTK_CONTAINER (vbox), spinner);
+
   if (rest_proxy_call_get_status_code (call) != 200)
     {
-      g_set_error (&data.error,
-                   GOA_ERROR,
-                   GOA_ERROR_FAILED,
-                   /* Translators: the %d is a HTTP status code and the %s is a textual description of it */
-                   _("Expected status 200 for getting a Request Token, instead got status %d (%s)"),
-                   rest_proxy_call_get_status_code (call),
-                   rest_proxy_call_get_status_message (call));
+      gchar *msg;
+
+      msg = goa_oauth_provider_parse_request_token_error (provider, call);
+      if (msg == NULL)
+        /* Translators: the %d is a HTTP status code and the %s is a textual description of it */
+        msg = g_strdup_printf (_("Expected status 200 for getting a Request Token, instead got status %d (%s)"),
+                               rest_proxy_call_get_status_code (call),
+                               rest_proxy_call_get_status_message (call));
+
+      g_set_error_literal (&data.error, GOA_ERROR, GOA_ERROR_FAILED, msg);
+      g_free (msg);
       goto out;
     }
   f = soup_form_decode (rest_proxy_call_get_payload (call));
@@ -702,46 +810,25 @@ get_tokens_and_identity (GoaOAuthProvider *provider,
     }
   else
     {
-      GtkWidget *scrolled_window;
       GtkWidget *web_view;
-      SoupSession *webkit_soup_session;
-      SoupCookieJar *cookie_jar;
+      GtkWidget *embed;
 
-      webkit_soup_session = webkit_get_default_session ();
-      /* Get the proxy configuration from the GNOME settings */
-      soup_session_add_feature_by_type (webkit_soup_session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
+      web_view = goa_web_view_new ();
+      embed = goa_web_view_get_view (GOA_WEB_VIEW (web_view));
 
-      /* Ensure we use an empty non-persistent cookie to avoid login
-       * credentials being reused...
-       */
-      soup_session_remove_feature_by_type (webkit_soup_session, SOUP_TYPE_COOKIE_JAR);
-      cookie_jar = soup_cookie_jar_new ();
-      soup_session_add_feature (webkit_soup_session, SOUP_SESSION_FEATURE (cookie_jar));
-      g_object_unref (cookie_jar);
-
-      /* TODO: we might need to add some more web browser UI to make this
-       * work...
-       */
-      web_view = webkit_web_view_new ();
-      webkit_web_view_load_uri (WEBKIT_WEB_VIEW (web_view), url);
-      g_signal_connect (web_view,
+      webkit_web_view_load_uri (WEBKIT_WEB_VIEW (embed), url);
+      g_signal_connect (embed,
+                        "document-load-finished",
+                        G_CALLBACK (on_web_view_document_load_finished),
+                        &data);
+      g_signal_connect (embed,
                         "navigation-policy-decision-requested",
                         G_CALLBACK (on_web_view_navigation_policy_decision_requested),
                         &data);
 
-      scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-      gtk_widget_set_size_request (scrolled_window, 500, 400);
-      gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
-                                           GTK_SHADOW_IN);
-      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-                                      GTK_POLICY_AUTOMATIC,
-                                      GTK_POLICY_AUTOMATIC);
-      gtk_container_add (GTK_CONTAINER (scrolled_window), web_view);
-      gtk_container_add (GTK_CONTAINER (vbox), scrolled_window);
-
-      gtk_widget_show_all (scrolled_window);
+      gtk_container_add (GTK_CONTAINER (vbox), web_view);
+      gtk_widget_show_all (web_view);
     }
-  data.dialog = dialog;
   gtk_dialog_run (GTK_DIALOG (dialog));
   if (data.oauth_verifier == NULL)
     {
@@ -910,8 +997,6 @@ goa_oauth_provider_add_account (GoaProvider *_provider,
   gint session_handle_expires_in;
   gchar *identity;
   gchar *presentation_identity;
-  GList *accounts;
-  GList *l;
   AddData data;
   GVariantBuilder builder;
 
@@ -927,7 +1012,6 @@ goa_oauth_provider_add_account (GoaProvider *_provider,
   session_handle = NULL;
   identity = NULL;
   presentation_identity = NULL;
-  accounts = NULL;
 
   memset (&data, '\0', sizeof (AddData));
   data.loop = g_main_loop_new (NULL, FALSE);
@@ -948,34 +1032,12 @@ goa_oauth_provider_add_account (GoaProvider *_provider,
   /* OK, got the identity... see if there's already an account
    * of this type with the given identity
    */
-  accounts = goa_client_get_accounts (client);
-  for (l = accounts; l != NULL; l = l->next)
-    {
-      GoaObject *object = GOA_OBJECT (l->data);
-      GoaAccount *account;
-      GoaOAuthBased *oauth_based;
-      const gchar *identity_from_object;
-
-      account = goa_object_peek_account (object);
-      oauth_based = goa_object_peek_oauth_based (object);
-      if (oauth_based == NULL)
-        continue;
-
-      if (g_strcmp0 (goa_account_get_provider_type (account),
-                     goa_provider_get_provider_type (GOA_PROVIDER (provider))) != 0)
-        continue;
-
-      identity_from_object = goa_account_get_identity (account);
-      if (g_strcmp0 (identity_from_object, identity) == 0)
-        {
-          g_set_error (&data.error,
-                       GOA_ERROR,
-                       GOA_ERROR_ACCOUNT_EXISTS,
-                       _("There is already an account for the identity %s"),
-                       identity);
-          goto out;
-        }
-    }
+  if (!goa_utils_check_duplicate (client,
+                                  identity,
+                                  goa_provider_get_provider_type (GOA_PROVIDER (provider)),
+                                  (GoaPeekInterfaceFunc) goa_object_peek_oauth_based,
+                                  &data.error))
+    goto out;
 
   /* we want the GoaClient to update before this method returns (so it
    * can create a proxy for the new object) so run the mainloop while
@@ -1018,18 +1080,14 @@ goa_oauth_provider_add_account (GoaProvider *_provider,
     goto out;
 
  out:
+  /* We might have an object even when data.error is set.
+   * eg., if we failed to store the credentials in the keyring.
+   */
   if (data.error != NULL)
-    {
-      g_propagate_error (error, data.error);
-      g_assert (ret == NULL);
-    }
+    g_propagate_error (error, data.error);
   else
-    {
-      g_assert (ret != NULL);
-    }
+    g_assert (ret != NULL);
 
-  g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
-  g_list_free (accounts);
   g_free (identity);
   g_free (presentation_identity);
   g_free (access_token);
@@ -1147,7 +1205,8 @@ goa_oauth_provider_refresh_account (GoaProvider  *_provider,
 static void
 free_mutex (GMutex *mutex)
 {
-  g_mutex_free (mutex);
+  g_mutex_clear (mutex);
+  g_slice_free (GMutex, mutex);
 }
 
 /**
@@ -1230,7 +1289,8 @@ goa_oauth_provider_get_access_token_sync (GoaOAuthProvider   *provider,
   lock = g_object_get_data (G_OBJECT (object), "-goa-oauth-provider-get-access-token-lock");
   if (lock == NULL)
     {
-      lock = g_mutex_new ();
+      lock = g_slice_new0 (GMutex);
+      g_mutex_init (lock);
       g_object_set_data_full (G_OBJECT (object),
                               "-goa-oauth-provider-get-access-token-lock",
                               lock,
@@ -1399,6 +1459,7 @@ goa_oauth_provider_build_object (GoaProvider         *provider,
                                  GoaObjectSkeleton   *object,
                                  GKeyFile            *key_file,
                                  const gchar         *group,
+                                 gboolean             just_added,
                                  GError             **error)
 {
   GoaOAuthBased *oauth_based;
