@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011 Red Hat, Inc.
+ * Copyright (C) 2011, 2012 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,15 +17,18 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * Author: David Zeuthen <davidz@redhat.com>
+ * Authors: David Zeuthen <davidz@redhat.com>
+ *          Debarshi Ray <debarshir@gnome.org>
  */
 
 #include "config.h"
+
 #include <glib/gi18n-lib.h>
 #include <gnome-keyring.h>
 
 #include "goalogging.h"
 #include "goaprovider.h"
+#include "goaexchangeprovider.h"
 #include "goagoogleprovider.h"
 #include "goafacebookprovider.h"
 #include "goayahooprovider.h"
@@ -52,13 +55,15 @@ static gboolean goa_provider_build_object_real (GoaProvider         *provider,
                                                 GoaObjectSkeleton   *object,
                                                 GKeyFile            *key_file,
                                                 const gchar         *group,
+                                                gboolean             just_added,
                                                 GError             **error);
 
 static void goa_provider_show_account_real (GoaProvider         *provider,
                                             GoaClient           *client,
                                             GoaObject           *object,
                                             GtkBox              *vbox,
-                                            GtkTable            *table);
+                                            GtkGrid             *left,
+                                            GtkGrid             *right);
 
 static guint goa_provider_get_credentials_generation_real (GoaProvider *provider);
 
@@ -182,7 +187,9 @@ goa_provider_get_provider_icon_default (GoaProvider *provider,
  * If an account was successfully created, a #GoaObject for the
  * created account is returned. If @dialog is dismissed, %NULL is
  * returned and @error is set to %GOA_ERROR_DIALOG_DISMISSED. If an
- * account couldn't be created then @error is set.
+ * account couldn't be created then @error is set. In some cases,
+ * for example, when the credentials could not be stored in the
+ * keyring, a #GoaObject can be returned even if @error is set.
  *
  * The caller will always show an error dialog if @error is set unless
  * the error is %GOA_ERROR_DIALOG_DISMISSED.
@@ -286,15 +293,17 @@ goa_provider_show_account (GoaProvider         *provider,
                            GoaClient           *client,
                            GoaObject           *object,
                            GtkBox              *vbox,
-                           GtkTable            *table)
+                           GtkGrid             *left,
+                           GtkGrid             *right)
 {
   g_return_if_fail (GOA_IS_PROVIDER (provider));
   g_return_if_fail (GOA_IS_CLIENT (client));
   g_return_if_fail (GOA_IS_OBJECT (object) && goa_object_peek_account (object) != NULL);
   g_return_if_fail (GTK_IS_BOX (vbox));
-  g_return_if_fail (GTK_IS_TABLE (table));
+  g_return_if_fail (GTK_IS_GRID (left));
+  g_return_if_fail (GTK_IS_GRID (right));
 
-  GOA_PROVIDER_GET_CLASS (provider)->show_account (provider, client, object, vbox, table);
+  GOA_PROVIDER_GET_CLASS (provider)->show_account (provider, client, object, vbox, left, right);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -304,9 +313,10 @@ goa_provider_show_account_real (GoaProvider         *provider,
                                 GoaClient           *client,
                                 GoaObject           *object,
                                 GtkBox              *vbox,
-                                GtkTable            *table)
+                                GtkGrid             *left,
+                                GtkGrid             *right)
 {
-  /* For now, don't do anything */
+  goa_util_add_account_info (left, right, object);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -317,6 +327,7 @@ goa_provider_show_account_real (GoaProvider         *provider,
  * @object: The #GoaObjectSkeleton that is being built.
  * @key_file: The #GKeyFile with configuation data.
  * @group: The group in @key_file to get data from.
+ * @just_added: Whether the account was newly created or being updated.
  * @error: Return location for error or %NULL.
  *
  * This method is called when construction account #GoaObject<!-- -->
@@ -340,6 +351,7 @@ goa_provider_build_object (GoaProvider         *provider,
                            GoaObjectSkeleton   *object,
                            GKeyFile            *key_file,
                            const gchar         *group,
+                           gboolean             just_added,
                            GError             **error)
 {
   g_return_val_if_fail (GOA_IS_PROVIDER (provider), FALSE);
@@ -347,7 +359,7 @@ goa_provider_build_object (GoaProvider         *provider,
   g_return_val_if_fail (key_file != NULL, FALSE);
   g_return_val_if_fail (group != NULL, FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-  return GOA_PROVIDER_GET_CLASS (provider)->build_object (provider, object, key_file, group, error);
+  return GOA_PROVIDER_GET_CLASS (provider)->build_object (provider, object, key_file, group, just_added, error);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -530,6 +542,7 @@ goa_provider_build_object_real (GoaProvider         *provider,
                                 GoaObjectSkeleton   *object,
                                 GKeyFile            *key_file,
                                 const gchar         *group,
+                                gboolean             just_added,
                                 GError             **error)
 {
   /* does nothing */
@@ -585,6 +598,9 @@ ensure_ep_and_builtins (void)
       extension_point = g_io_extension_point_register (GOA_PROVIDER_EXTENSION_POINT_NAME);
       g_io_extension_point_set_required_type (extension_point, GOA_TYPE_PROVIDER);
 
+#ifdef GOA_EXCHANGE_ENABLED
+      type = GOA_TYPE_EXCHANGE_PROVIDER;
+#endif
 #ifdef GOA_GOOGLE_ENABLED
       type = GOA_TYPE_GOOGLE_PROVIDER;
 #endif
@@ -904,7 +920,7 @@ goa_provider_delete_credentials_sync (GoaProvider   *provider,
       g_set_error (error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED, /* TODO: more specific */
-                   "Failed to delete credentials from the keyring: %s",
+                   _("Failed to delete credentials from the keyring: %s"),
                    gnome_keyring_result_to_message (result));
       goto out;
     }
@@ -926,9 +942,20 @@ goa_provider_delete_credentials_sync (GoaProvider   *provider,
  * Various utility routines.
  */
 
+/* ---------------------------------------------------------------------------------------------------- */
+
+gchar *
+goa_util_get_css (void)
+{
+  return g_strdup (PACKAGE_DATA_DIR "/" PACKAGE "/" PACKAGE ".css");
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 /**
  * goa_util_add_row_widget:
- * @table: A #GtkTable.
+ * @left: A #GtkGrid for the left side.
+ * @right: A #GtkGrid for the right side.
  * @label_text: (allow-none): The text to insert on the left side or %NULL for no label.
  * @widget: A widget to insert on the right side.
  *
@@ -937,62 +964,30 @@ goa_provider_delete_credentials_sync (GoaProvider   *provider,
  * Returns: (transfer none): The #GtkWidget that was inserted (e.g. @widget itself).
  */
 GtkWidget *
-goa_util_add_row_widget (GtkTable     *table,
+goa_util_add_row_widget (GtkGrid      *left,
+                         GtkGrid      *right,
                          const gchar  *label_text,
                          GtkWidget    *widget)
 {
   GtkWidget *label;
-  guint num_rows;
 
-  g_return_val_if_fail (GTK_IS_TABLE (table), NULL);
+  g_return_val_if_fail (GTK_IS_GRID (left), NULL);
+  g_return_val_if_fail (GTK_IS_GRID (right), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
-  gtk_table_get_size (table, &num_rows, NULL);
-
   if (label_text != NULL)
     {
-      gchar *s;
-      s = g_strdup_printf ("<span foreground=\"#555555\">%s</span>", label_text);
-      label = gtk_label_new (NULL);
-      gtk_label_set_markup (GTK_LABEL (label), s);
-      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-      gtk_table_attach (table, label,
-                        0, 1,
-                        num_rows, num_rows + 1,
-                        GTK_FILL, GTK_FILL, 0, 0);
-      g_free (s);
-    }
-  gtk_table_attach (table, widget,
-                    1, 2,
-                    num_rows, num_rows + 1,
-                    GTK_FILL, GTK_FILL, 0, 0);
-  return widget;
-}
+      GtkStyleContext *context;
 
-/**
- * goa_util_add_row_label:
- * @table: A #GtkTable.
- * @label_text: (allow-none): The text to insert on the left side or %NULL for no label.
- * @value_markup: The markup to insert on the right side.
- *
- * Utility function to add @label_text and @value_text to @table.
- *
- * Returns: (transfer none): The #GtkLabel that was inserted.
- */
-GtkWidget *
-goa_util_add_row_label (GtkTable     *table,
-                        const gchar  *label_text,
-                        const gchar  *value_markup)
-{
-  GtkWidget *label;
-  label = gtk_label_new (NULL);
-  gtk_label_set_markup (GTK_LABEL (label), value_markup);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_widget_set_margin_top (label, 6);
-  gtk_widget_set_margin_bottom (label, 6);
-  gtk_widget_set_margin_left (label, 6);
-  gtk_label_set_selectable (GTK_LABEL (label), TRUE);
-  return goa_util_add_row_widget (table, label_text, label);
+      label = gtk_label_new (label_text);
+      context = gtk_widget_get_style_context (label);
+      gtk_style_context_add_class (context, "dim-label");
+      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+      gtk_container_add (GTK_CONTAINER (left), label);
+    }
+
+  gtk_container_add (GTK_CONTAINER (right), widget);
+  return widget;
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1103,44 +1098,18 @@ goa_util_lookup_keyfile_boolean (GoaObject    *object,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-typedef struct {
-  GoaObject *object;
-  gchar *key;
-} KeyFileEditableData;
-
-static KeyFileEditableData *
-keyfile_editable_data_new (GoaObject   *object,
-                           const gchar *key)
-{
-  KeyFileEditableData *data;
-  data = g_slice_new (KeyFileEditableData);
-  data->object = g_object_ref (object);
-  data->key = g_strdup (key);
-  return data;
-}
-
 static void
-keyfile_editable_data_free (KeyFileEditableData *data)
+goa_util_set_keyfile_boolean (GoaAccount *account, const gchar *key, gboolean value)
 {
-  g_object_unref (data->object);
-  g_free (data->key);
-  g_slice_free (KeyFileEditableData, data);
-}
-
-static void
-keyfile_editable_on_editing_done (GtkEditable  *editable,
-                                  gpointer      user_data)
-{
-  KeyFileEditableData *data = user_data;
-  GoaAccount *account;
   GError *error;
   GKeyFile *key_file;
-  gchar *path;
-  gchar *group;
   gchar *contents;
+  gchar *group;
+  gchar *path;
   gsize length;
 
-  account = goa_object_peek_account (data->object);
+  contents = NULL;
+
   path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
   group = g_strdup_printf ("Account %s", goa_account_get_id (account));
 
@@ -1153,488 +1122,129 @@ keyfile_editable_on_editing_done (GtkEditable  *editable,
     {
       goa_warning ("Error loading keyfile %s: %s (%s, %d)",
                    path,
-                   error->message, g_quark_to_string (error->domain), error->code);
+                   error->message,
+                   g_quark_to_string (error->domain),
+                   error->code);
       g_error_free (error);
       goto out;
     }
 
-  g_key_file_set_string (key_file,
-                         group,
-                         data->key,
-                         goa_editable_label_get_text (GOA_EDITABLE_LABEL (editable)));
+  g_key_file_set_boolean (key_file, group, key, value);
+  contents = g_key_file_to_data (key_file, &length, NULL);
 
   error = NULL;
-  contents = g_key_file_to_data (key_file,
-                                 &length,
-                                 &error);
-  if (contents == NULL)
+  if (!g_file_set_contents (path, contents, length, &error))
     {
-      g_prefix_error (&error,
-                      "Error generating key-value-file %s: ",
-                      path);
-      goa_warning ("%s (%s, %d)",
-                   error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-
-  error = NULL;
-  if (!g_file_set_contents (path,
-                            contents,
-                            length,
-                            &error))
-    {
-      g_prefix_error (&error,
-                      "Error writing key-value-file %s: ",
-                      path);
-      goa_warning ("%s (%s, %d)",
-                   error->message, g_quark_to_string (error->domain), error->code);
+      g_prefix_error (&error, "Error writing key-value-file %s: ", path);
+      goa_warning ("%s (%s, %d)", error->message, g_quark_to_string (error->domain), error->code);
       g_error_free (error);
       goto out;
     }
 
  out:
+  g_free (contents);
   g_key_file_free (key_file);
   g_free (group);
   g_free (path);
 }
 
-/**
- * goa_util_add_row_editable_label_from_keyfile:
- * @table: A #GtkTable.
- * @object: A #GoaObject for an account.
- * @label_text: (allow-none): The text to insert on the left side or %NULL for no label.
- * @key: The key in the key-value file for @object to look up.
- * @editable: Whether the widget should be editable
- *
- * Adds a #GoaEditableLabel to @table that reads its value from the
- * key-value file for @object using @key. If it's edited, the new
- * value is written back to the key-value file.
- *
- * Returns: (transfer none): The #GoaEditableLabel that was inserted.
- */
-GtkWidget *
-goa_util_add_row_editable_label_from_keyfile (GtkTable     *table,
-                                              GoaObject    *object,
-                                              const gchar  *label_text,
-                                              const gchar  *key,
-                                              gboolean      editable)
+void
+goa_util_account_notify_property_cb (GObject *object, GParamSpec *pspec, gpointer user_data)
 {
   GoaAccount *account;
-  GtkWidget *elabel;
-  GKeyFile *key_file;
-  gchar *path;
-  gchar *group;
-  GError *error;
-  gchar *value;
+  gboolean value;
+  const gchar *key;
+  const gchar *name;
 
-  key_file = NULL;
-  value = NULL;
+  g_return_if_fail (GOA_IS_ACCOUNT (object));
 
-  account = goa_object_peek_account (object);
-  elabel = goa_editable_label_new ();
-  path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
-  group = g_strdup_printf ("Account %s", goa_account_get_id (account));
+  account = GOA_ACCOUNT (object);
+  key = user_data;
+  name = g_param_spec_get_name (pspec);
 
-  key_file = g_key_file_new ();
-  error = NULL;
-  if (!g_key_file_load_from_file (key_file,
-                                  path,
-                                  G_KEY_FILE_NONE,
-                                  &error))
-    {
-      goa_warning ("Error loading keyfile %s: %s (%s, %d)",
-                   path,
-                   error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-  value = g_key_file_get_string (key_file,
-                                 group,
-                                 key,
-                                 &error);
-  if (value == NULL)
-    {
-      /* this is not fatal (think upgrade-path) */
-      goa_debug ("Error getting value for key %s in group `%s' from keyfile %s: %s (%s, %d)",
-                 key,
-                 group,
-                 path,
-                 error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-
-  goa_editable_label_set_text (GOA_EDITABLE_LABEL (elabel), value);
-
- out:
-
-  if (editable)
-    {
-      goa_editable_label_set_editable (GOA_EDITABLE_LABEL (elabel), TRUE);
-      g_signal_connect_data (elabel,
-                             "editing-done",
-                             G_CALLBACK (keyfile_editable_on_editing_done),
-                             keyfile_editable_data_new (object, key),
-                             (GClosureNotify) keyfile_editable_data_free,
-                             0); /* GConnectFlags */
-    }
-
-  g_free (value);
-  g_free (group);
-  g_free (path);
-  if (key_file != NULL)
-    g_key_file_free (key_file);
-  return goa_util_add_row_widget (table, label_text, elabel);
+  g_object_get (account, name, &value, NULL);
+  goa_util_set_keyfile_boolean (account, key, !value);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static void
-keyfile_switch_on_notify_active (GObject      *object,
-                                 GParamSpec   *pspec,
-                                 gpointer      user_data)
+void
+goa_util_add_account_info (GtkGrid *left, GtkGrid *right, GoaObject *object)
 {
-  KeyFileEditableData *data = user_data;
+  GIcon *icon;
   GoaAccount *account;
-  GError *error;
-  GKeyFile *key_file;
-  gchar *path;
-  gchar *group;
-  gchar *contents;
-  gsize length;
+  GtkWidget *image;
+  GtkWidget *label;
+  const gchar *icon_str;
+  const gchar *identity;
+  const gchar *name;
+  gchar *markup;
 
-  account = goa_object_peek_account (data->object);
-  path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
-  group = g_strdup_printf ("Account %s", goa_account_get_id (account));
+  account = goa_object_peek_account (object);
 
-  key_file = g_key_file_new ();
-  error = NULL;
-  if (!g_key_file_load_from_file (key_file,
-                                  path,
-                                  G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS,
-                                  &error))
-    {
-      goa_warning ("Error loading keyfile %s: %s (%s, %d)",
-                   path,
-                   error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
+  icon_str = goa_account_get_provider_icon (account);
+  icon = g_icon_new_for_string (icon_str, NULL);
+  image = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_DIALOG);
+  g_object_unref (icon);
+  gtk_misc_set_alignment (GTK_MISC (image), 1.0, 0.5);
+  gtk_widget_set_margin_bottom (image, 12);
+  gtk_container_add (GTK_CONTAINER (left), image);
 
-  g_key_file_set_boolean (key_file,
-                          group,
-                          data->key,
-                          gtk_switch_get_active (GTK_SWITCH (object)));
+  name = goa_account_get_provider_name (account);
+  identity = goa_account_get_presentation_identity (account);
+  markup = g_strdup_printf ("<b>%s</b>\n%s",
+                            name,
+                            (identity == NULL || identity[0] == '\0') ? "\xe2\x80\x94" : identity);
+  label = gtk_label_new (NULL);
+  gtk_label_set_markup (GTK_LABEL (label), markup);
+  gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+  gtk_label_set_max_width_chars (GTK_LABEL (label), 24);
+  gtk_label_set_width_chars (GTK_LABEL (label), 24);
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_widget_set_margin_bottom (label, 12);
+  g_free (markup);
+  gtk_container_add (GTK_CONTAINER (right), label);
 
-  error = NULL;
-  contents = g_key_file_to_data (key_file,
-                                 &length,
-                                 &error);
-  if (contents == NULL)
-    {
-      g_prefix_error (&error,
-                      "Error generating key-value-file %s: ",
-                      path);
-      goa_warning ("%s (%s, %d)",
-                   error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-
-  error = NULL;
-  if (!g_file_set_contents (path,
-                            contents,
-                            length,
-                            &error))
-    {
-      g_prefix_error (&error,
-                      "Error writing key-value-file %s: ",
-                      path);
-      goa_warning ("%s (%s, %d)",
-                   error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-
- out:
-  g_key_file_free (key_file);
-  g_free (group);
-  g_free (path);
+  return;
 }
 
-/**
- * goa_util_add_row_switch_from_keyfile:
- * @table: A #GtkTable.
- * @object: A #GoaObject for an account.
- * @label_text: (allow-none): The text to insert on the left side or %NULL for no label.
- * @key: The key in the key-value file for @object to look up.
- *
- * Adds a #GtkSwitch to @table that reads its #GtkSwitch:active value
- * from the key-value file for @object using @key. If it's switched,
- * the new value is written back to the key-value file.
- *
- * Returns: (transfer none): The #GtkSwitch that was inserted.
- */
+/* ---------------------------------------------------------------------------------------------------- */
+
 GtkWidget *
-goa_util_add_row_switch_from_keyfile (GtkTable     *table,
-                                      GoaObject    *object,
-                                      const gchar  *label_text,
-                                      const gchar  *key)
+goa_util_add_row_switch_from_keyfile_with_blurb (GtkGrid      *left,
+                                                 GtkGrid      *right,
+                                                 GoaObject    *object,
+                                                 const gchar  *label_text,
+                                                 const gchar  *property,
+                                                 const gchar  *blurb)
 {
   GoaAccount *account;
   GtkWidget *hbox;
   GtkWidget *switch_;
-  GKeyFile *key_file;
-  gchar *path;
-  gchar *group;
-  GError *error;
   gboolean value;
 
-  key_file = NULL;
-
   account = goa_object_peek_account (object);
+  g_object_get (account, property, &value, NULL);
   switch_ = gtk_switch_new ();
-  path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
-  group = g_strdup_printf ("Account %s", goa_account_get_id (account));
+  gtk_switch_set_active (GTK_SWITCH (switch_), !value);
 
-  key_file = g_key_file_new ();
-  error = NULL;
-  if (!g_key_file_load_from_file (key_file,
-                                  path,
-                                  G_KEY_FILE_NONE,
-                                  &error))
+  g_object_bind_property (switch_, "active",
+                          account, property,
+                          G_BINDING_BIDIRECTIONAL | G_BINDING_INVERT_BOOLEAN);
+
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_set_homogeneous (GTK_BOX (hbox), FALSE);
+
+  if (blurb != NULL)
     {
-      goa_warning ("Error loading keyfile %s: %s (%s, %d)",
-                   path,
-                   error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-  value = g_key_file_get_boolean (key_file,
-                                  group,
-                                  key,
-                                  &error);
-  if (error != NULL)
-    {
-      /* this is not fatal (think upgrade-path) */
-      goa_debug ("Error getting boolean value for key %s in group `%s' from keyfile %s: %s (%s, %d)",
-                 key,
-                 group,
-                 path,
-                 error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
+      GtkWidget *label;
+
+      label = gtk_label_new (blurb);
+      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+      gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
     }
 
-  gtk_switch_set_active (GTK_SWITCH (switch_), value);
-
- out:
-  g_signal_connect_data (switch_,
-                         "notify::active",
-                         G_CALLBACK (keyfile_switch_on_notify_active),
-                         keyfile_editable_data_new (object, key),
-                         (GClosureNotify) keyfile_editable_data_free,
-                         0); /* GConnectFlags */
-  if (key_file != NULL)
-    g_key_file_free (key_file);
-  g_free (group);
-  g_free (path);
-
-  hbox = gtk_hbox_new (0, FALSE);
-  gtk_box_pack_start (GTK_BOX (hbox), switch_, FALSE, TRUE, 0);
-  goa_util_add_row_widget (table, label_text, hbox);
+  gtk_box_pack_end (GTK_BOX (hbox), switch_, FALSE, FALSE, 0);
+  goa_util_add_row_widget (left, right, label_text, hbox);
   return switch_;
 }
-
-/* ---------------------------------------------------------------------------------------------------- */
-
-static void
-keyfile_check_button_on_notify_active (GObject      *object,
-                                       GParamSpec   *pspec,
-                                       gpointer      user_data)
-{
-  KeyFileEditableData *data = user_data;
-  GoaAccount *account;
-  GError *error;
-  GKeyFile *key_file;
-  gchar *path;
-  gchar *group;
-  gchar *contents;
-  gsize length;
-
-  account = goa_object_peek_account (data->object);
-  path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
-  group = g_strdup_printf ("Account %s", goa_account_get_id (account));
-
-  key_file = g_key_file_new ();
-  error = NULL;
-  if (!g_key_file_load_from_file (key_file,
-                                  path,
-                                  G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS,
-                                  &error))
-    {
-      goa_warning ("Error loading keyfile %s: %s (%s, %d)",
-                   path,
-                   error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-
-  g_key_file_set_boolean (key_file,
-                          group,
-                          data->key,
-                          gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (object)));
-
-  error = NULL;
-  contents = g_key_file_to_data (key_file,
-                                 &length,
-                                 &error);
-  if (contents == NULL)
-    {
-      g_prefix_error (&error,
-                      "Error generating key-value-file %s: ",
-                      path);
-      goa_warning ("%s (%s, %d)",
-                   error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-
-  error = NULL;
-  if (!g_file_set_contents (path,
-                            contents,
-                            length,
-                            &error))
-    {
-      g_prefix_error (&error,
-                      "Error writing key-value-file %s: ",
-                      path);
-      goa_warning ("%s (%s, %d)",
-                   error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-
- out:
-  g_key_file_free (key_file);
-  g_free (group);
-  g_free (path);
-}
-
-/**
- * goa_util_add_row_check_button_from_keyfile:
- * @table: A #GtkTable.
- * @object: A #GoaObject for an account.
- * @label_text: (allow-none): The text to insert on the left side or %NULL for no label.
- * @key: The key in the key-value file for @object to look up.
- * @value_mnemonic: The mnemonic text to use for the check button.
- *
- * Adds a #GtkCheckButton to @table that reads its value from the
- * key-value file for @object using @key. If it's toggled, the new
- * value is written back to the key-value file.
- *
- * Returns: (transfer none): The #GtkCheckButton that was inserted.
- */
-GtkWidget *
-goa_util_add_row_check_button_from_keyfile (GtkTable     *table,
-                                            GoaObject    *object,
-                                            const gchar  *label_text,
-                                            const gchar  *key,
-                                            const gchar  *value_mnemonic)
-{
-  GoaAccount *account;
-  GtkWidget *check_button;
-  GKeyFile *key_file;
-  gchar *path;
-  gchar *group;
-  GError *error;
-  gboolean value;
-
-  key_file = NULL;
-
-  account = goa_object_peek_account (object);
-  check_button = gtk_check_button_new_with_mnemonic (value_mnemonic);
-  path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
-  group = g_strdup_printf ("Account %s", goa_account_get_id (account));
-
-  key_file = g_key_file_new ();
-  error = NULL;
-  if (!g_key_file_load_from_file (key_file,
-                                  path,
-                                  G_KEY_FILE_NONE,
-                                  &error))
-    {
-      goa_warning ("Error loading keyfile %s: %s (%s, %d)",
-                   path,
-                   error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-  value = g_key_file_get_boolean (key_file,
-                                  group,
-                                  key,
-                                  &error);
-  if (error != NULL)
-    {
-      /* this is not fatal (think upgrade-path) */
-      goa_debug ("Error getting boolean value for key %s in group `%s' from keyfile %s: %s (%s, %d)",
-                 key,
-                 group,
-                 path,
-                 error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button), value);
-
- out:
-  g_signal_connect_data (check_button,
-                         "notify::active",
-                         G_CALLBACK (keyfile_check_button_on_notify_active),
-                         keyfile_editable_data_new (object, key),
-                         (GClosureNotify) keyfile_editable_data_free,
-                         0); /* GConnectFlags */
-  if (key_file != NULL)
-    g_key_file_free (key_file);
-  g_free (group);
-  g_free (path);
-
-  return goa_util_add_row_widget (table, label_text, check_button);
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
-/**
- * goa_util_add_heading:
- * @table: A #GtkTable.
- * @heading_text: The text for the heading.
- *
- * Utility function to add a heading to @table.
- *
- * Returns: (transfer none): The #GtkWidget that was inserted.
- */
-GtkWidget *
-goa_util_add_heading (GtkTable     *table,
-                      const gchar  *heading_text)
-{
-  GtkWidget *label;
-  guint num_rows;
-  gchar *s;
-
-  gtk_table_get_size (table, &num_rows, NULL);
-  s = g_strdup_printf ("<b>%s</b>", heading_text);
-  label = gtk_label_new (NULL);
-  gtk_label_set_markup (GTK_LABEL (label), s);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_widget_set_margin_top (label, 6);
-  gtk_widget_set_margin_bottom (label, 6);
-  gtk_table_attach (table, label,
-                    0, 1,
-                    num_rows, num_rows + 1,
-                    GTK_FILL, GTK_FILL, 0, 0);
-  g_free (s);
-
-  return label;
-}
