@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011 Red Hat, Inc.
+ * Copyright (C) 2011, 2012 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,8 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * Author: David Zeuthen <davidz@redhat.com>
+ * Authors: David Zeuthen <davidz@redhat.com>
+ *          Debarshi Ray <debarshir@gnome.org>
  */
 
 #if !defined (__GOA_BACKEND_INSIDE_GOA_BACKEND_H__) && !defined (GOA_BACKEND_COMPILATION)
@@ -92,6 +93,7 @@ struct _GoaProviderClass
                                      GoaObjectSkeleton  *object,
                                      GKeyFile           *key_file,
                                      const gchar        *group,
+                                     gboolean            just_added,
                                      GError            **error);
 
   /* virtual but with default implementation */
@@ -104,7 +106,8 @@ struct _GoaProviderClass
                                        GoaClient           *client,
                                        GoaObject           *object,
                                        GtkBox              *vbox,
-                                       GtkTable            *table);
+                                       GtkGrid             *left,
+                                       GtkGrid             *right);
   guint    (*get_credentials_generation) (GoaProvider   *provider);
 
   /*< private >*/
@@ -132,11 +135,13 @@ void         goa_provider_show_account              (GoaProvider         *provid
                                                      GoaClient           *client,
                                                      GoaObject           *object,
                                                      GtkBox              *vbox,
-                                                     GtkTable            *table);
+                                                     GtkGrid             *left,
+                                                     GtkGrid             *right);
 gboolean     goa_provider_build_object              (GoaProvider         *provider,
                                                      GoaObjectSkeleton   *object,
                                                      GKeyFile            *key_file,
                                                      const gchar         *group,
+                                                     gboolean             just_added,
                                                      GError             **error);
 gboolean     goa_provider_store_credentials_sync    (GoaProvider         *provider,
                                                      GoaObject           *object,
@@ -180,15 +185,12 @@ GoaProvider  *goa_provider_get_for_provider_type (const gchar *provider_type);
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-GtkWidget *goa_util_add_heading (GtkTable     *table,
-                                 const gchar  *heading_text);
-
-GtkWidget *goa_util_add_row_widget (GtkTable     *table,
+GtkWidget *goa_util_add_row_widget (GtkGrid      *left,
+                                    GtkGrid      *right,
                                     const gchar  *label_text,
                                     GtkWidget    *widget);
-GtkWidget *goa_util_add_row_label  (GtkTable     *table,
-                                    const gchar  *label_text,
-                                    const gchar  *value_markup);
+
+gchar     *goa_util_get_css        (void);
 
 gchar *
 goa_util_lookup_keyfile_string (GoaObject    *object,
@@ -198,24 +200,17 @@ gboolean
 goa_util_lookup_keyfile_boolean (GoaObject    *object,
                                  const gchar  *key);
 
-GtkWidget *goa_util_add_row_editable_label_from_keyfile (GtkTable     *table,
-                                                         GoaObject    *object,
-                                                         const gchar  *label_text,
-                                                         const gchar  *key,
-                                                         gboolean      editable);
+void
+goa_util_account_notify_property_cb (GObject *object, GParamSpec *pspec, gpointer user_data);
 
-GtkWidget *goa_util_add_row_switch_from_keyfile (GtkTable     *table,
-                                                 GoaObject    *object,
-                                                 const gchar  *label_text,
-                                                 const gchar  *key);
-
-GtkWidget *
-goa_util_add_row_check_button_from_keyfile (GtkTable     *table,
-                                            GoaObject    *object,
-                                            const gchar  *label_text,
-                                            const gchar  *key,
-                                            const gchar  *value_mnemonic);
+void       goa_util_add_account_info (GtkGrid *left, GtkGrid *right, GoaObject *object);
 
+GtkWidget *goa_util_add_row_switch_from_keyfile_with_blurb (GtkGrid      *left,
+                                                            GtkGrid      *right,
+                                                            GoaObject    *object,
+                                                            const gchar  *label_text,
+                                                            const gchar  *key,
+                                                            const gchar  *blurb);
 
 G_END_DECLS
 
